<?php
/**
* 2007-2019 PrestaShop
*
* NOTICE OF LICENSE
*
* This source file is subject to the Academic Free License (AFL 3.0)
* that is bundled with this package in the file LICENSE.txt.
* It is also available through the world-wide-web at this URL:
* http://opensource.org/licenses/afl-3.0.php
* If you did not receive a copy of the license and are unable to
* obtain it through the world-wide-web, please send an email
* to license@prestashop.com so we can send you a copy immediately.
*
* DISCLAIMER
*
* Do not edit or add to this file if you wish to upgrade PrestaShop to newer
* versions in the future. If you wish to customize PrestaShop for your
* needs please refer to http://www.prestashop.com for more information.
*
*  @author    PrestaShop SA <contact@prestashop.com>
*  @copyright 2007-2019 PrestaShop SA
*  @license   http://opensource.org/licenses/afl-3.0.php  Academic Free License (AFL 3.0)
*  International Registered Trademark & Property of PrestaShop SA
*/

if (!defined('_PS_VERSION_')) {
    exit;
}

class Data8validation extends Module
{
    protected $config_form = false;

    public function __construct()
    {
        $this->name = 'data8validation';
        $this->tab = 'administration';
        $this->version = '2.0.2';
        $this->author = 'Data8';
        $this->need_instance = 0;

        /**
         * Set $this->bootstrap to true if your module is compliant with bootstrap (PrestaShop 1.6)
         */
        $this->bootstrap = true;

        parent::__construct();

        $this->displayName = $this->l('Data 8 Data Capture and Validation');
        $this->description = $this->l('Real-time data validation for your PrestaShop store to improve customer experience and reduce errors in customer data entry.');

        $this->confirmUninstall = $this->l('Are you sure that you want to uninstall? All user configuration will be lost.');

        $this->ps_versions_compliancy = array('min' => '1.6', 'max' => _PS_VERSION_);
    }

    /**
     * Don't forget to create update methods if needed:
     * http://doc.prestashop.com/display/PS16/Enabling+the+Auto-Update
     */
    public function install()
    {
        Configuration::updateValue('Data8_apiKey', null);
        Configuration::updateValue('Data8_usePredictiveAddress', null);
        Configuration::updateValue('Data8_useW3WLocation', null);
        Configuration::updateValue('Data8_useOnBlur', null);
        Configuration::updateValue('Data8_emailValidationLevel', null);
        Configuration::updateValue('Data8_useTelephoneValidation', null);
        Configuration::updateValue('Data8_defaultCountryCode', null);
        Configuration::updateValue('Data8_useSalaciousName', null);

        include(dirname(__FILE__).'/sql/install.php');

        return parent::install() &&
            $this->registerHook('header') &&
            $this->registerHook('displayfooter') &&
            $this->registerHook('backOfficeHeader');
    }

    public function uninstall()
    {
        Configuration::deleteByName('Data8_apiKey');
        Configuration::deleteByName('Data8_usePredictiveAddress');
        Configuration::deleteByName('Data8_useOnBlur');
        Configuration::deleteByName('Data8_emailValidationLevel');
        Configuration::deleteByName('Data8_useTelephoneValidation');
        Configuration::deleteByName('Data8_defaultCountryCode');
        Configuration::deleteByName('Data8_useSalaciousName');

        include(dirname(__FILE__).'/sql/uninstall.php');

        return parent::uninstall();
    }

    /**
     * Load the configuration form
     */
    public function getContent()
    {
        $output = null;

        /**
         * If values have been submitted in the form, process.
         */
        if (((bool)Tools::isSubmit('submitData8validationModule')) == true) {
            $this->postProcess();
            $output .= $this->displayConfirmation($this->l('Settings updated'));
        }

        $this->context->smarty->assign('module_dir', $this->_path);

        $output .= $this->context->smarty->fetch($this->local_path.'views/templates/admin/configure.tpl');

        return $output.$this->renderForm();
    }

    /**
     * Create the form that will be displayed in the configuration of your module.
     */
    protected function renderForm()
    {
        $helper = new HelperForm();

        $helper->show_toolbar = false;
        $helper->table = $this->table;
        $helper->module = $this;
        $helper->default_form_language = $this->context->language->id;
        $helper->allow_employee_form_lang = Configuration::get('PS_BO_ALLOW_EMPLOYEE_FORM_LANG', 0);

        $helper->identifier = $this->identifier;
        $helper->submit_action = 'submitData8validationModule';
        $helper->currentIndex = $this->context->link->getAdminLink('AdminModules', false)
            .'&configure='.$this->name.'&tab_module='.$this->tab.'&module_name='.$this->name;
        $helper->token = Tools::getAdminTokenLite('AdminModules');

        $helper->tpl_vars = array(
            'fields_value' => $this->getConfigFormValues(), /* Add values for your inputs */
            'languages' => $this->context->controller->getLanguages(),
            'id_language' => $this->context->language->id,
        );

        return $helper->generateForm(array($this->getConfigForm()));
    }

    /**
     * Create the structure of your form.
     */
    protected function getConfigForm()
    {
        return array(
            'form' => array(
                'legend' => array(
                'title' => "&nbsp; ".$this->l('Settings'),
                'icon' => 'icon-cogs',
                ),
                'input' => array(
                    array(
                        'col' => 3,
                        'type' => 'text',
                        'prefix' => '<i class="icon icon-key"></i>',
                        'desc' => $this->l("Enter your Data8 Client-Side Api Key found on your Data8 Dashboard. ").'<strong>'.$this->l("Don't have an account yet? ")."<a href='https://www.data-8.co.uk/ValidationAdmin/FreeTrial/ContactForm7' target='_blank'>".$this->l('Sign up for a free trial now')."</a></strong>.",
                        'name' => 'Data8_apiKey',
                        'label' => '<strong>'.$this->l('Data8 Api Key').'</strong>',
                    ),
                    array(
                        'type' => 'switch',
                        'label' => '<strong>'.$this->l('Use Data8 ').'<a href="https://www.data-8.co.uk/data-validation/address-validation/" target="_blank">'.$this->l('PredictiveAddress').'&trade;</a></strong>',
                        'name' => 'Data8_usePredictiveAddress',
                        'is_bool' => true,
                        'desc' => $this->l('Use auto-complete for addresses.'),
                        'values' => array(
                            array(
                                'id' => 'active_on',
                                'value' => true,
                                'label' => $this->l('Enabled')
                            ),
                            array(
                                'id' => 'active_off',
                                'value' => false,
                                'label' => $this->l('Disabled')
                            )
                        ),
                    ),
                    array(
                        'col' => 3,
                        'type' => 'switch',
                        'desc' => $this->l("Retreive the exact latitude and longitude using what3words (requires additional licensing)."),
                        'name' => 'Data8_useW3WLocation',
                        'label' => '<strong>'.$this->l('Use what3words Location').'</strong>',
                        'is_bool' => true,
                        'values' => array(
                            array(
                                'id' => 'active_on',
                                'value' => true,
                                'label' => $this->l('Enabled')
                            ),
                            array(
                                'id' => 'active_off',
                                'value' => false,
                                'label' => $this->l('Disabled')
                            )
                        ),
                    ),
                    array(
                        'type' => 'select',
                        'desc' => $this->l('Validate fields on focus change (onBlur) rather than on form submission (onSubmit).'),
                        'name' => 'Data8_useOnBlur',
                        'label' => '<strong>'.$this->l('Validation Method').'</strong>',
                        'options' => array(
                            'query' => $onBlur_options = array(
                                array(
                                    'onBlur_options' => true,
                                    'name' => 'On focus change'
                                ),
                                array(
                                    'onBlur_options' => false,
                                    'name' => 'On form submission'
                                ),
                            ),
                            'id' => 'onBlur_options',
                            'name' => 'name',
                        ),
                    ),
                    array(
                        'type' => 'select',
                        'desc' => $this->l("Select the level to validate email addersses to. Higher validation levels give better accuracy but take longer."),
                        'name' => 'Data8_emailValidationLevel',
                        'label' => '<strong>'.$this->l('Data8 ').'<a href="https://www.data-8.co.uk/data-validation/email-validation/" target="_blank">'.$this->l('Email Validation').'</a>'.$this->l(' level').'</strong>',
                        'options' => array(
                            'query' => $email_options = array(
                                array(
                                    'email_options' => '',
                                    'name' => 'None'
                                ),
                                array(
                                    'email_options' => 'Syntax',
                                    'name' => 'Syntax'
                                ),
                                array(
                                    'email_options' => 'MX',
                                    'name' => 'Domain'
                                ),
                                array(
                                    'email_options' => 'Server',
                                    'name' => 'Server'
                                ),
                                array(
                                    'email_options' => 'Address',
                                    'name' => 'Address (Highest)'
                                ),
                            ),
                            'id' => 'email_options',
                            'name' => 'name',
                        ),
                    ),
                    array(
                        'type' => 'switch',
                        'label' => '<strong>'.$this->l('Use Data8 ').'<a href="https://www.data-8.co.uk/services/data-validation/validation-services/phone-validation" target="_blank">'.$this->l('Phone Validation').'</a></strong>',
                        'name' => 'Data8_useTelephoneValidation',
                        'is_bool' => true,
                        'desc' => $this->l('Validate international telephone numbers.'),
                        'values' => array(
                            array(
                                'id' => 'active_on',
                                'value' => true,
                                'label' => $this->l('Enabled')
                            ),
                            array(
                                'id' => 'active_off',
                                'value' => false,
                                'label' => $this->l('Disabled')
                            )
                        ),
                    ),
                    array(
                        'col' => 3,
                        'type' => 'text',
                        'desc' => $this->l("Country code to assume when validating telephone numbers, e.g. 44 for UK, 1 for US. Any telephone numbers for other countries will need to be fully specified including the +(country code) prefix in order to be correctly validated."),
                        'name' => 'Data8_defaultCountryCode',
                        'label' => '<strong>'.$this->l('Default Telephone Country Code').'</strong>',
                    ),
                    array(
                        'type' => 'switch',
                        'label' => '<strong>'.$this->l('Use Data8 Unusable Name Validation').'</strong>',
                        'name' => 'Data8_useSalaciousName',
                        'is_bool' => true,
                        'desc' => $this->l('Validate name fields.'),
                        'values' => array(
                            array(
                                'id' => 'active_on',
                                'value' => true,
                                'label' => $this->l('Enabled')
                            ),
                            array(
                                'id' => 'active_off',
                                'value' => false,
                                'label' => $this->l('Disabled')
                            )
                        ),
                    )
                ),
                'submit' => array(
                    'title' => $this->l('Save'),
                ),
            ),
        );
    }

    /**
     * Set values for the inputs.
     */
    protected function getConfigFormValues()
    {
        return array(
            'Data8_apiKey' => Configuration::get('Data8_apiKey', 'Your-Api-Key-Here'),
            'Data8_usePredictiveAddress' => Configuration::get('Data8_usePredictiveAddress', false),
            'Data8_useW3WLocation' => Configuration::get('Data8_useW3WLocation', false),
            'Data8_useOnBlur' => Configuration::get('Data8_useOnBlur', false),
            'Data8_emailValidationLevel' => Configuration::get('Data8_emailValidationLevel', 'MX'),
            'Data8_useTelephoneValidation' => Configuration::get('Data8_useTelephoneValidation', false),
            'Data8_defaultCountryCode' => Configuration::get('Data8_defaultCountryCode', '44'),
            'Data8_useSalaciousName' => Configuration::get('Data8_useSalaciousName', false),
        );
    }

    /**
     * Save form data.
     */
    protected function postProcess()
    {
        $form_values = $this->getConfigFormValues();

        foreach (array_keys($form_values) as $key) {
            Configuration::updateValue($key, Tools::getValue($key));
        }
    }

    /**
    * Add the CSS & JavaScript files you want to be loaded in the BO.
    */
    public function hookBackOfficeHeader()
    {
        if(substr(_PS_VERSION_, 0, 3) === "1.7"){
            $this->context->controller->addJS($this->_path.'views/js/back.js');
            //$this->context->controller->registerJavascript('module-data8validation-back', 'modules/' . $this->name .'views/js/back.js', ['priority' => 200, 'attribute' => 'async']);
        }
        else if (substr(_PS_VERSION_, 0, 3) === "1.6"){
            $this->context->controller->addJS($this->_path.'views/js/back.js');
        }
    }

    /**
     * Add the CSS & JavaScript files you want to be added on the FO.
     */
    public function hookHeader()
    {
        if(substr(_PS_VERSION_, 0, 3) === "1.7"){
            $this->context->controller->registerJavascript('module-data8validation-front', $this->_path.'/views/js/front.js', ['priority' => 200, 'attribute' => 'async']);
            $this->context->controller->registerStylesheet('module-data8validation-style', $this->_path.'/views/css/front.css',['media' => 'all', 'priority' => 200]);

            Media::addJsDef( array(
                'data8_psVersion' => substr(_PS_VERSION_, 0, 3),
                'data8_apiKey' => Configuration::get('Data8_apiKey'),
                'data8_usePredictiveAddress' => Configuration::get('Data8_usePredictiveAddress') ? 'true' : 'false',
                'data8_useW3WLocation' => Configuration::get('Data8_useW3WLocation') ? 'true' : 'false',
                'data8_useOnBlur' => Configuration::get('Data8_useOnBlur') ? 'true' : 'false',
                'data8_emailValidationLevel' => Configuration::get('Data8_emailValidationLevel'),
                'data8_useTelephoneValidation' => Configuration::get('Data8_useTelephoneValidation') ? 'true' : 'false',
                'data8_defaultCountryCode' => Configuration::get('Data8_defaultCountryCode'),
                'data8_useSalaciousName' => Configuration::get('Data8_useSalaciousName') ? 'true' : 'false',
            ));
        }
    }

    public function hookDisplayFooter(array $params){
        
            if (substr(_PS_VERSION_, 0, 3) === "1.6"){
                $this->context->controller->addJS($this->_path.'/views/js/front.js');
                $this->context->controller->addCSS($this->_path.'/views/css/front.css');

                Media::addJsDef( array(
                    'data8_psVersion' => substr(_PS_VERSION_, 0, 3),
                    'data8_apiKey' => Configuration::get('Data8_apiKey'),
                    'data8_usePredictiveAddress' => Configuration::get('Data8_usePredictiveAddress') ? 'true' : 'false',
                    'data8_useW3WLocation' => Configuration::get('Data8_useW3WLocation') ? 'true' : 'false',
                    'data8_useOnBlur' => Configuration::get('Data8_useOnBlur') ? 'true' : 'false',
                    'data8_emailValidationLevel' => Configuration::get('Data8_emailValidationLevel'),
                    'data8_useTelephoneValidation' => Configuration::get('Data8_useTelephoneValidation') ? 'true' : 'false',
                    'data8_defaultCountryCode' => Configuration::get('Data8_defaultCountryCode'),
                    'data8_useSalaciousName' => Configuration::get('Data8_useSalaciousName') ? 'true' : 'false',
                ));
            }
        //$this->context->controller->registerJavascript('module-data8validation-data8Url', "http://localhost:3747/Javascript/prestashop-2.js", ['server' => 'remote', 'position' => 'bottom', 'priority' => 150]);
    }
}
