/**
* 2007-2019 PrestaShop
*
* NOTICE OF LICENSE
*
* This source file is subject to the Academic Free License (AFL 3.0)
* that is bundled with this package in the file LICENSE.txt.
* It is also available through the world-wide-web at this URL:
* http://opensource.org/licenses/afl-3.0.php
* If you did not receive a copy of the license and are unable to
* obtain it through the world-wide-web, please send an email
* to license@prestashop.com so we can send you a copy immediately.
*
* DISCLAIMER
*
* Do not edit or add to this file if you wish to upgrade PrestaShop to newer
* versions in the future. If you wish to customize PrestaShop for your
* needs please refer to http://www.prestashop.com for more information.
*
*  @author    PrestaShop SA <contact@prestashop.com>
*  @copyright 2007-2019 PrestaShop SA
*  @license   http://opensource.org/licenses/afl-3.0.php  Academic Free License (AFL 3.0)
*  International Registered Trademark & Property of PrestaShop SA
*
* Don't forget to prefix your containers with your own identifier
* to avoid any conflicts with others containers.
*/

(function () {
    $(document).ready(function () {
        // The only jQuery now remaining in the document are this document.ready and triggering change events in fireChangeEvents().
        if (!window.data8_apiKey)
            return;

        // Functions to Load Scripts and Stylesheets into prestashop theme.
        var loadScript = function (url, callback) {
            var dst = document.createElement("script");
            dst.src = url;
            dst.onload = callback;
            document.getElementsByTagName("head")[0].appendChild(dst);
        }

        var addStyle = function (url) {
            var dst = document.createElement("link");
            dst.rel = "stylesheet"
            dst.href = url;
            document.getElementsByTagName("head")[0].appendChild(dst);
        }

        // Check to ensure code is only ran once
        if (!window.data8_PredictiveAddressHasAttatched) {

            window.data8_PredictiveAddressHasAttatched = true;

            // Attach Predictive Address and/or Form Validation on form submit.
            if (window.data8_usePredictiveAddress == "true") {

                addStyle('https://webservices.data-8.co.uk/content/predictiveaddress.css');
                loadScript("https://webservices.data-8.co.uk/javascript/predictiveaddress.js", function () {
                    data8_prestashopAttachValidation();
                });
            }
            else {

                data8_prestashopAttachValidation();
            }
        }

        function data8_prestashopAttachValidation() {
           
            var data8_isElementVisible = function (el) {
                return !!(el.offsetWidth || el.offsetHeight || el.getClientRects().length);
            };

            var data8_getElement = function (idOrName) {
                var element = document.getElementById(idOrName);

                if (element)
                    return element;

                var elements = document.getElementsByName(idOrName);
                var visible = null;

                for (var i = 0; i < elements.length; i++) {
                    if (data8_isElementVisible(elements[i])) {
                        if (visible)
                            return null;

                        visible = elements[i];
                    }
                }

                return visible;
            }

            var webservicesDomain = "https://webservices.data-8.co.uk"; //"http://localhost:3747";

            // fields to attach predictive address to.
            var addressFields = [
                { element: "company", field: "organisation" },
                { element: "address1", field: "line1" },
                { element: "address2", field: "line2" },
                { element: "city", field: "town" },
                { element: "id_state", field: "county" },
                { element: "postcode", field: "postcode" },
                { element: "id_country", field: "country" }
            ];

            // Array of elements to try and attach PredictiveAddress data entry to
            var data8PredictiveAddressAttachTo = [
                { attachTo: "company", fields: addressFields },
                { attachTo: "address1", fields: addressFields }
            ];

            var latitudeClass = "data8_latitude";
            var longitudeClass = "data8_longitude";
            var w3wClass = "data8_what3words";

            // special case function to convert US addresses into PrestaShop validation friendly results.
            function data8_predictiveAddressUSSpecialCase(fields, address) {
                // Special case for US:
                // 1. Remove the +4 from the zip code to pass the standard Prestashop zip code validation
                for (var i = 0; i < fields.length; i++) {
                    if (fields[i].field === "postcode") {
                        var postcodeElement = data8_getElement(fields[i].element);
                        var zipCode = postcodeElement.value;
                        zipCode = zipCode.replace(/^(\d{5})-\d{4}$/, "$1");
                        postcodeElement.value = zipCode;
                        break;
                    }
                }

                // 2. Convert state abbreviation to name and select option by text
                var states = {
                    "AL": "Alabama",
                    "AK": "Alaska",
                    "AZ": "Arizona",
                    "AR": "Arkansas",
                    "CA": "California",
                    "CO": "Colorado",
                    "CT": "Connecticut",
                    "DE": "Delaware",
                    "DC": "District of Columbia",
                    "FL": "Florida",
                    "GA": "Georgia",
                    "HI": "Hawaii",
                    "ID": "Idaho",
                    "IL": "Illinois",
                    "IN": "Indiana",
                    "IA": "Iowa",
                    "KS": "Kansas",
                    "KY": "Kuntucky",
                    "LA": "Louisiana",
                    "ME": "Maine",
                    "MD": "Maryland",
                    "MA": "Massachusetts",
                    "MI": "Michigan",
                    "MN": "Minnesota",
                    "MS": "Mississippi",
                    "MO": "Missouri",
                    "MT": "Montana",
                    "NE": "Nebraska",
                    "NV": "Nevada",
                    "NH": "New Hampshire",
                    "NJ": "New Jersey",
                    "NM": "New Mexico",
                    "NY": "New York",
                    "NC": "North Carolina",
                    "ND": "North Dakota",
                    "OH": "Ohio",
                    "OK": "oklahoma",
                    "OR": "Oregon",
                    "PA": "Pennsylvania",
                    "RI": "Rhode Island",
                    "SC": "South Carolina",
                    "SD": "South Dakota",
                    "TN": "Tennessee",
                    "TX": "Texas",
                    "UT": "Utah",
                    "VT": "Vermont",
                    "VA": "Virginia",
                    "WA": "Washington",
                    "WV": "West Virginia",
                    "WI": "Wisconsin",
                    "WY": "Wyoming",
                    "PR": "Puerto Rico",
                    "VI": "US Vrgin Islands"
                };

                var stateName = states[address.RawAddress.PostalCounty];

                if (stateName) {
                    for (var i = 0; i < fields.length; i++) {
                        if (fields[i].field === "county") {
                            var stateElement = data8_getElement(fields[i].element);

                            for (var j = 0; j < stateElement.options.length; j++) {
                                if (stateElement.options[j].text === stateName) {
                                    stateElement.selectedIndex = j;
                                    break;
                                }
                            }

                            break;
                        }
                    }
                }
            }

            function data8_predictiveAddressAppendLocation(result) {
                var latitudeField;
                var longitudeField;
                var w3wField;
            
                if (latitudeClass && longitudeClass) {
                    latitudeField = document.querySelector('.' + latitudeClass);
                    longitudeField = document.querySelector('.' + longitudeClass);
                }
            
                if (latitudeField && longitudeField && result.RawAddress.Location) {
                    latitudeField.value = result.RawAddress.Location.Latitude;
                    longitudeField.value = result.RawAddress.Location.Longitude;
                }
            
                if (w3wClass && result.RawAddress.AdditionalData.length > 0) {
                    w3wField = document.querySelector('.' + w3wClass);
                    if (!w3wField) return;
            
                    for (var index = 0; index < result.RawAddress.AdditionalData.length; index++) {
                        var data = result.RawAddress.AdditionalData[index];
                        if (data.Name === "What3Words") {
                            w3wField.value = data.Value;
                            break;
                        }
                    }
                }
            }

            // function to attach predictive address to the 'AttachTo' fields in the form
            var data8_canLoadAddress = function () {
                if (window.data8_usePredictiveAddress) {
                    for (var i = 0; i < data8PredictiveAddressAttachTo.length; i++) {
                        var fields = data8PredictiveAddressAttachTo[i].fields;
                        var textbox = data8_getElement(data8PredictiveAddressAttachTo[i].attachTo);
                        if (textbox == null)
                            continue;

                        if (!data8_getElement(fields[5].element))
                            continue;

                        var useW3W = false;
                        if(window.data8_useW3WLocation == "true"){
                            useW3W = true;
                        }

                        new data8.predictiveaddressui(textbox, {
                            ajaxKey: window.data8_apiKey,
                            fields: fields,
                            initialCountry: "auto",
                            countryFieldFormat: "name",
                            includeWhat3Words: useW3W,
                            applicationName: "PrestaShop",
                            selectAddress: function (address, country) {
                                if (country === "United States")
                                    data8_predictiveAddressUSSpecialCase(fields, address);

                                data8_predictiveAddressAppendLocation(address);
                            }
                        });
                    }
                }
            };

            var CallsInProgress = []; //keep track of calls in progress to halt form submission until all async calls have finished
            var _d8isValid; //bool to keep track of whether any fields are invalid.

            function data8_attachValidation() {
                    
                if (window.data8_useOnBlur && window.data8_useOnBlur === "true") {
                    // if useOnBlur enabled then it needs to applied to each input field individually.
                    var forms = document.forms;
                    for (var i = 0; i < forms.length; i++) {
                        data8_validateFields(forms[i], null);
                    }

                    // on submit, perform a check to ensure no fields are in error before submitting
                    document.getElementsByTagName('body')[0].addEventListener('submit', function (e) {
                        var submitElem = event.target;
                        if (submitElem.tagName === 'FORM') {
                            var data8ErrorMessages = submitElem.getElementsByClassName("data8-error");
                            if (CallsInProgress.length <= 0 && data8ErrorMessages.length > 0) {
                                e.preventDefault();
                                e.stopImmediatePropagation();
                            }
                            else{
                                setTimeout(function(){
                                    if(window.location.hash == "#account-creation"){
                                        if (window.data8_useOnBlur && window.data8_useOnBlur === "true") {
                                            // if useOnBlur enabled then it needs to applied to each input field individually.
                                            var forms = document.forms;
                                            for (var i = 0; i < forms.length; i++) {
                                                data8_validateFields(forms[i], null);
                                            }
                                        }
                                    }
                                }, 2000);
                            }
                        }
                    });
                }
                else {
                    // attach to onSubmit event of form. Uses delegated event to ensure it's attached after partial page reload
                    document.getElementsByTagName('body')[0].addEventListener('submit', function (e) {
                        var submitElem = event.target;
                        if (submitElem.tagName === 'FORM') {
                            data8_validateFields(submitElem, e);
                        }
                    });
                }

                data8_canLoadAddress();
            }

            // function to loop through all input fields in the form that was submitted, and attempt to validate the fields
            function data8_validateFields(_form, _event) {
                _d8isValid = true;

                // Validate / Add listener events to all suitable input fields
                var inputFields = _form.getElementsByTagName("input");
                for (var i = 0; i < inputFields.length; i++) {
                    var _field = inputFields[i];
                    if (_field.getAttribute("type") != "hidden") {

                        // Email Validation
                        if (window.data8_emailValidationLevel && window.data8_emailValidationLevel !== "None") {
                            if (_field.name.indexOf("email") !== -1 || _field.getAttribute("type").indexOf("email") !== -1) {
                                // if onBlur then apply event listener on page ready
                                if (window.data8_useOnBlur && window.data8_useOnBlur === "true") {
                                    _field.addEventListener('blur', function () {
                                        data8_emailVal(this);
                                    });
                                }
                                // else begin validation on the field as page has been submitted
                                else {
                                    if (_field.value)
                                        data8_emailVal(_field);
                                }
                            }
                        }

                        // International Telephone Validation
                        if (window.data8_useTelephoneValidation == "true") {
                            if (_field.name.indexOf("phone") !== -1 || _field.getAttribute("type").indexOf("tel") !== -1) {
                                if (window.data8_useOnBlur && window.data8_useOnBlur === "true") {
                                    _field.addEventListener('blur', function () {
                                        data8_phoneVal(this);
                                    });
                                }
                                else {
                                    if (_field.value)
                                        data8_phoneVal(_field);
                                }
                            }
                        }

                        // Salacious Name Validation
                        if (window.data8_useSalaciousName == "true") {
                            if (_field.name.indexOf("name") !== -1 && _field.name.indexOf("username") === -1) {
                                if (window.data8_useOnBlur && window.data8_useOnBlur === "true") {
                                    _field.addEventListener('blur', function () {
                                        data8_nameVal(this);
                                    });
                                }
                                else {
                                    if (_field.value)
                                        data8_nameVal(_field);
                                }
                            }
                        }
                    }
                }

                // Check that it is validating at least one field before stopping default actions
                if (CallsInProgress.length > 0) {
                    if (_event) {
                        _event.preventDefault();
                        _event.stopImmediatePropagation();
                    }
                }
            }

            // functions to validate each type of field respectively
            function data8_emailVal(_emailField) {
                if (_emailField) {
                    if (_emailField.value) {
                        _emailField.setAttribute('d8val_email', 'true');
                        _emailField.setAttribute('d8validationlevel', window.data8_emailValidationLevel);

                        var fieldArray = [];
                        fieldArray.push(_emailField);
                        var params = JSON.stringify({
                            "email": _emailField.value,
                            "level": window.data8_emailValidationLevel,
                            "options": {
                                "ApplicationName": "PrestaShop"
                            }
                        });

                        var url = webservicesDomain + "/EmailValidation/IsValid.json?key=" + window.data8_apiKey;
                        CallsInProgress.push(_emailField);
                        data8_asyncRequest(url, params, data8_handleValidation, fieldArray, "email address");
                    }
                    else {
                        data8_clearError(_emailField);
                    }
                }
            }

            function data8_phoneVal(_telField) {
                if (_telField) {
                    if (_telField.value) {
                        var _countryCode = window.data8_defaultCountryCode;
                        if(_countryCode)
                            _countryCode = _countryCode.trim();

                        if(!_countryCode || _countryCode === "")
                            _countryCode = "44";

                        _telField.setAttribute('d8val_inttelephone', 'true');
                        _telField.setAttribute('d8defCountry', _countryCode);

                        var fieldArray = [];
                        fieldArray.push(_telField);
                        var params = JSON.stringify({
                            "telephoneNumber": _telField.value,
                            "defaultCountry": _countryCode,
                            "options": {
                                "ApplicationName": "PrestaShop"
                            }
                        });

                        var url = webservicesDomain + "/PhoneValidation/IsValid.json?key=" + window.data8_apiKey;
                        CallsInProgress.push(_telField);
                        data8_asyncRequest(url, params, data8_handleValidation, fieldArray, "telephone number");
                    }
                    else {
                        data8_clearError(_telField);
                    }
                }
            }

            function data8_nameVal(_nameField) {
                var bValidateName = true;
                var _form = _nameField.form;
                if (_nameField) {
                    if (_nameField.value) {
                        _nameField.setAttribute('d8val_salaciousName', 'true');
                        var firstNameValue = _nameField.value;
                        var lastNameValue = "";
                        var fieldArray = [];
                        fieldArray.push(_nameField);

                        // add unique class to first and last name fields
                        if (_nameField.name.indexOf("first") !== -1 || _nameField.name.indexOf("fore") !== -1) {
                            _nameField.classList.add("d8val_firstName");
                            bValidateName = false;

                            firstNameValue = _nameField.value;
                            var lastNameArr = _form.getElementsByClassName('d8val_lastName');
                            if (lastNameArr.length > 0 && CallsInProgress.indexOf(lastNameArr[0]) === -1) {
                                lastNameValue = lastNameArr[0].value;
                                fieldArray.push(lastNameArr[0]);
                                bValidateName = true;
                            }
                            else if(window.data8_useOnBlur && window.data8_useOnBlur === "true") {
                                var lastNameField = jQuery("input[name*='last'][name*='name']");
                                if(lastNameField && lastNameField.length > 0){ lastNameField = lastNameField[0]; }
                                if(lastNameField && lastNameField.value){
                                    fieldArray.push(lastNameField);
                                    bValidateName = true;
                                    lastNameField.classList.add("d8val_lastName");
                                    lastNameField.setAttribute('d8val_salaciousName', 'true');
                                    lastNameValue = lastNameField.value;
                                }
                            }
                        }
                        else if (_nameField.name.indexOf("last") !== -1 || _nameField.name.indexOf("sur") !== -1) {
                            _nameField.classList.add("d8val_lastName");
                            bValidateName = false;

                            lastNameValue = _nameField.value;
                            var firstNameArr = _form.getElementsByClassName('d8val_firstName');
                            if (firstNameArr.length > 0 && CallsInProgress.indexOf(firstNameArr[0]) === -1) {
                                firstNameValue = firstNameArr[0].value;
                                fieldArray.push(firstNameArr[0]);
                                bValidateName = true;
                            }
                            else if(window.data8_useOnBlur && window.data8_useOnBlur === "true") {
                                var firstNameField = jQuery("input[name*='first'][name*='name']");
                                if(firstNameField && firstNameField.length > 0){ firstNameField = firstNameField[0]; }
                                if(firstNameField && firstNameField.value){
                                    fieldArray.push(firstNameField);
                                    bValidateName = true;
                                    firstNameField.classList.add("d8val_firstName");
                                    firstNameField.setAttribute('d8val_salaciousName', 'true');
                                    firstNameValue = firstNameField.value;
                                }
                            }
                        }

                        if (bValidateName) {
                            var params = JSON.stringify({
                                "name": {
                                    "Title": "",
                                    "Forename": firstNameValue,
                                    "MiddleName": "",
                                    "Surname": lastNameValue
                                },
                                "options": {
                                    "ApplicationName": "PrestaShop"
                                }
                            });

                            var url = webservicesDomain + "/SalaciousName/IsUnusableName.json?key=" + window.data8_apiKey;
                            CallsInProgress.push(_nameField);
                            data8_asyncRequest(url, params, data8_handleValidation, fieldArray, "name");                                    
                        }
                    }
                    else {
                        data8_clearError(_nameField);

                        // find other name field if it is likely to exist and clear errors on that also
                        var otherFieldArray;
                        if (_nameField.name.indexOf("firstname") !== -1) {
                            otherFieldArray = _form.getElementsByClassName('d8val_lastName');
                        }
                        else if (_nameField.name.indexOf("lastname") !== -1) {
                            otherFieldArray = _form.getElementsByClassName('d8val_firstName');
                        }

                        if (otherFieldArray && otherFieldArray.length > 0) {
                            data8_clearError(otherFieldArray[0]);
                        }
                    }
                }
            }

            //function to handle the result of any validation requests
            function data8_handleValidation(result, fieldArray, errorString) {
                if (result) {
                    if (!result.Status.Success) { return; }

                    // Get submit button of form
                    var d8form = fieldArray[0].form;
                    var fieldCount = fieldArray.length;
                    // perform error handling for each field passed in validation check
                    // could be used to output an error only on one field of address for example if postcode wasn't valid?
                    for (var j = 0; j < fieldArray.length; j++) {
                        var element = fieldArray[j];
                        // Delete existing error messages
                        data8_clearError(element);

                        var hasErrored = false;
                        if (element.getAttribute("d8val_inttelephone")) {
                            if (result.Result.ValidationResult == "Invalid" || result.Result.ValidationResult == "Unavailable") {
                                hasErrored = true;
                            }
                        }
                        else if (element.getAttribute("d8val_email")) {
                            if (result.Result != "Valid" && result.Result != "CatchAll") {
                                hasErrored = true;
                            }
                        }
                        else if (element.getAttribute("d8val_salaciousName")) {
                            if (result.Result != "") {
                                hasErrored = true;
                            }
                        }

                        // remove this element from the list of calls still in progress
                        if (CallsInProgress.indexOf(element) != -1) { CallsInProgress.splice(CallsInProgress.indexOf(element), 1); }
                        fieldCount--;
                        if (hasErrored) {
                            var errorMsg = 'Please enter a valid ' + errorString + "."; // error message

                            //insert errorMsg into html as a sibling of email element (child of email element parent)
                            var parent = element.parentNode;

                            if (parent) {

                                var node = window.document.createElement("SPAN")
                                node.innerHTML = errorMsg;
                                node.classList.add("data8-error");
                                parent.insertBefore(node, element.nextSibling);
                            }

                            if(window.data8_useOnBlur && window.data8_useOnBlur === "true" && window.data8_psVersion == "1.7"){
                                element.setCustomValidity(errorMsg);
                            }
                            else if(window.data8_psVersion == "1.6"){
                                element.closest(".form-group").classList.add("form-error");
                                element.closest(".form-group").classList.remove("form-ok");
                            }

                            _d8isValid = false;
                        }
                        else {
                            if (!window.data8_useOnBlur || window.data8_useOnBlur === "false") {
                                if (CallsInProgress.length <= 0 && fieldCount <= 0) {
                                    if (_d8isValid){
                                        if(window.data8_psVersion === "1.6" && d8form.id === 'create-account_form'){
                                            window.submitFunction();
                                        }
                                        else{
                                            d8form.submit();
                                        }
                                    }
                                }
                            }
                        }
                    }
                }
            }

            // function to perform the async request to our services
            function data8_asyncRequest(serviceUrl, data, callbackFunction, fieldArray, errorString) {
                //Creating a new XMLHttpRequest object
                var request;
                if (window.XMLHttpRequest) {
                    request = new XMLHttpRequest(); //for IE7+, Firefox, Chrome, Opera, Safari
                } else {
                    request = new ActiveXObject("Microsoft.XMLHTTP"); //for IE6, IE5
                }

                //When readyState is 4 then get the server output
                request.onreadystatechange = function () {
                    if (request.readyState == 4) {
                        if (request.status == 200) {
                            // parse data to make it useable
                            callbackFunction(JSON.parse(request.responseText), fieldArray, errorString);
                        }
                    }
                };

                //Create a asynchronous POST request
                request.open("POST", serviceUrl, true);

                request.send(data);
            }

            // function to clear any existing errors on form re-submission before re-validating
            function data8_clearError(_field) {
                var errorElem = _field.parentNode.querySelector(".data8-error");
                if (errorElem) {
                    _field.parentNode.removeChild(errorElem); 
                }

                if(window.data8_useOnBlur && window.data8_useOnBlur === "true" && window.data8_psVersion == "1.7"){
                    _field.setCustomValidity("");
                }
                else if(window.data8_psVersion == "1.6"){
                    if (_field.closest(".form-group").classList.contains("form-error")) {
                        _field.closest(".form-group").classList.remove("form-error");
                    }
                    // add positive validation feedback
                    _field.closest(".form-group").classList.add("form-ok");
                }
            }

            data8_attachValidation();
        }
    });
})();
